//
//  HeroEditController.m
//  SuperDB
//
//  Created by Ken Anderson on 3/16/11.
//  Copyright 2011 University of Colorado, Boulder. All rights reserved.
//

#import "HeroEditController.h"
#import "NSArray-NestedArrays.h"
#import "HeroValueDisplay.h"
#import "ManagedObjectAttributeEditor.h"

@implementation HeroEditController

@synthesize hero=_hero;

- (void)viewWillAppear:(BOOL)animated {	
    [self.tableView reloadData];
    [super viewWillAppear:animated];
}

- (void) viewDidLoad {
    
    sectionNames = [[NSArray alloc] initWithObjects:[NSNull null], @"General", nil];
    
    rowLabels    = [[NSArray alloc] initWithObjects:
                    
                    [NSArray arrayWithObjects:@"Name", nil],
                    
                    [NSArray arrayWithObjects:@"Identity", @"Birthdate", @"Sex", nil],
                    
                    nil];
    
    rowKeys      = [[NSArray alloc] initWithObjects:
                    
                    [NSArray arrayWithObjects:@"name", nil],
                    
                    [NSArray arrayWithObjects:@"secretIdentity", @"birthdate", @"sex", nil],
                    
                    nil];
    
    rowControllers = [[NSArray alloc] initWithObjects:
                      
                      [NSArray arrayWithObject:@"ManagedObjectStringEditor"],
                      
                      [NSArray arrayWithObjects:@"ManagedObjectStringEditor", @"ManagedObjectDateEditor", @"ManagedObjectSingleSelectionListEditor", nil],
                      
                      nil];
    
    rowArguments = [[NSArray alloc] initWithObjects:
                    
                    [NSArray arrayWithObject:[NSNull null]],
                    
                    [NSArray arrayWithObjects:
                       [NSNull null],
                       [NSNull null],
                       [NSDictionary dictionaryWithObject:[NSArray arrayWithObjects:@"Male", @"Female", nil] forKey:@"list"], nil],
                    
                    nil];

    [super viewDidLoad];

}

- (void) dealloc {
    [_hero release];
    [sectionNames release];
    [rowLabels release];
    [rowKeys release];
    [rowControllers release];
    [super dealloc];
}

#pragma mark -
#pragma mark Table View Methods

- (NSInteger) numberOfSectionsInTableView:(UITableView *)tableView {
    return [sectionNames count];
}

- (NSString *) tableView:(UITableView *)tableView titleForHeaderInSection:(NSInteger)section {
    id theTitle = [sectionNames objectAtIndex:section];
    
    if ([theTitle isKindOfClass:[NSNull class]]) {
        return nil;
    }
    
    return (NSString *) theTitle;
    
}

- (NSInteger) tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section {
    return [rowLabels countOfNestedArray:section];
}

- (UITableViewCell *) tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath {
    
    static NSString *HeroEditCellIdentifier = @"Hero Edit Cell Identifier";
    
    UITableViewCell *cell = [tableView dequeueReusableCellWithIdentifier:HeroEditCellIdentifier];
    if (cell == nil) {
        cell = [[[UITableViewCell alloc] initWithStyle:UITableViewCellStyleValue2 reuseIdentifier:HeroEditCellIdentifier] autorelease];
    }
    
    NSString *rowKey   = [rowKeys nestedObjectAtIndexPath:indexPath];
    NSString *rowLabel = [rowLabels nestedObjectAtIndexPath:indexPath];
    
    id <HeroValueDisplay> rowValue = [self.hero valueForKey:rowKey];
    
    cell.detailTextLabel.text = [rowValue heroValueDisplay];
    cell.textLabel.text = rowLabel;
    cell.accessoryType = UITableViewCellAccessoryDisclosureIndicator;
    
    return cell;
}

- (void) tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath {
    
    NSString *controllerClassName = [rowControllers nestedObjectAtIndexPath:indexPath];
    
    NSString *rowLabel = [rowLabels nestedObjectAtIndexPath:indexPath];
    NSString *rowKey   = [rowKeys nestedObjectAtIndexPath:indexPath];
    
    Class controllerClass = NSClassFromString(controllerClassName);
    ManagedObjectAttributeEditor *controller = [controllerClass alloc];
    
    controller = [controller initWithStyle:UITableViewStyleGrouped];
    controller.keypath = rowKey;
    controller.managedObject = self.hero;
    controller.labelString = rowLabel;
    controller.title = rowLabel;
    
    NSDictionary *args = [rowArguments nestedObjectAtIndexPath:indexPath];
    if ([args isKindOfClass:[NSDictionary class]]) {
        if (args != nil) {
            for (NSString *key in args) {
                id arg = [args objectForKey:key];
                [controller setValue:arg forKey:key];
            }
        }
    }
    
    [self.navigationController pushViewController:controller animated:YES];
    [controller release];
    
}

@end
